package com.exquisiteapplicationsolutions.tis.activities;

import android.app.AlertDialog;
import android.content.ContentValues;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.os.Bundle;
import android.os.Handler;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;

import com.exquisiteapplicationsolutions.tis.DBLogs;
import com.exquisiteapplicationsolutions.tis.DBParameters;
import com.exquisiteapplicationsolutions.tis.DateConvertClass;
import com.exquisiteapplicationsolutions.tis.R;
import com.exquisiteapplicationsolutions.tis.TISDBHelper;
import com.exquisiteapplicationsolutions.tis.jobRowArrayAdapter;
import com.exquisiteapplicationsolutions.tis.objects.JobRow;
import com.exquisiteapplicationsolutions.tis.syncClasses.TISCommon;

import org.json.JSONArray;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Locale;


public class JobsCenter extends AppCompatActivity {

    private static final String sDBName = "TIS.db";
    private static final int iVersionVal = 1;
    private static String sAndroidID = "";
    private static String sUserID = "";
    private static ArrayList<ArrayList<String>> jobArray;
    private Handler oHandler = new Handler();
    private static String sClosedDays = "2"; //days of closed jobs to show

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_jobs_center);

        SharedPreferences prefs = getSharedPreferences("TISSharedPrefs",
                Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = prefs.edit();
        String sLoggedInUserCompany = prefs.getString("LoggedInUserCompany", "No Value");
        sUserID = prefs.getString("UserID", "No Value");
        TextView otvUserName = (TextView) findViewById(R.id.tvUserName);
        otvUserName.setText(sLoggedInUserCompany.substring(0,sLoggedInUserCompany.indexOf('-')));

        String sCurrentVersion = prefs.getString("TISVersion", "No Value");
        TextView otvCopyright = (TextView) findViewById(R.id.tvCopyright);
        String sVersionText = otvCopyright.getText().toString();
        // need to replace the %RELNUM% with the actual rel version
        sVersionText = sVersionText.replace("RELNUM", sCurrentVersion);
        otvCopyright.setText(sVersionText);

        Button btnReqJob = (Button) findViewById(R.id.btnReqJob);
        btnReqJob.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                try {
                    InputMethodManager imm = (InputMethodManager) getSystemService(INPUT_METHOD_SERVICE);
                    imm.hideSoftInputFromWindow(getCurrentFocus().getWindowToken(), 0);
                } catch (Exception e) {
                    // TODO: handle exception
                }
                reqJobs();
                getJobsFromDB();
            }
        });
        Button btnCreateJob = (Button) findViewById(R.id.btnCreateJob);
        btnCreateJob.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                try {
                    if (TISCommon.isOnline(getApplicationContext())) {
                        Intent createJobIntent = new Intent(JobsCenter.this, CreateJob.class);
                        startActivity(createJobIntent);
                    } else {
                        String sTitle = getResources().getString(R.string.create_job_error_must_be_online);
                        String sMessage = getResources().getString(R.string.create_job_error_must_be_online_msg);
                        errorDialogue(sTitle, sMessage);
                    }
                } catch (Exception e) {
                    // TODO: handle exception
                }
            }
        });
        oHandler.post(timedTask);
    }

    @Override
    public void onResume()
    {
        super.onResume();
        updateUI();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_jobs_center, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }
        if (id == R.id.action_logout) {
            finish();
            Intent i = new Intent(getApplicationContext(), TISActivity.class);
            startActivity(i);
            return true;
        }
        if (id == R.id.action_icon_meanings) {
            Intent i = new Intent(getApplicationContext(), HelpActivity.class);
            i.putExtra("ScreenID", "IconMeanings");
            startActivity(i);
            return true;
        }
        if (id == R.id.action_help) {

             // display a full page
            Intent i = new Intent(getApplicationContext(), HelpActivity.class);
            i.putExtra("ScreenID", "JobsCenter");
            startActivity(i);

            return true;
        }
        if (id == R.id.action_about) {
            Intent i = new Intent(getApplicationContext(), AboutActivity.class);
            startActivity(i);
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    private Runnable timedTask = new Runnable() {
        @Override
        public void run() {
            // TODO Auto-generated method stub
            updateUI();
            oHandler.postDelayed(timedTask, 20000);
        }
    };

    /*****************************************************************************************/
    /* This method will get the number of available jobs */

    /**
     * *************************************************************************************
     */
    private void getJobsFromDB() {
        TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName,
                null, iVersionVal);
        try {
            oDB.createDataBase();
            oDB.openDataBase();
            String sLanguage = Locale.getDefault().getLanguage();
            String sSQL = "";
            switch (sLanguage) {
                case "en":
                    sSQL = "select A.job_id, A.date_time, A.job_status_id, B.name_en, A.assigned_id, A.manifest_num";
                    sSQL += ", A.container_num, A.delivery_address_line1, A.delivery_address_line2, A.delivery_city";
                    sSQL += ", A.delivery_state, A.delivery_postal_code, A.pickup_address_line1, A.pickup_address_line2";
                    sSQL += ", A.pickup_city, A.pickup_state, A.pickup_postal_code, A.username";
                    sSQL += ", A.importer_id, A.importer_name, A.job_num, A.pickup_date_time, A.delivery_date_time";
                    sSQL += ", A.ref_note from jobs as A join job_status_type as B on A.job_status_id = B.id order by A.date_time desc";
                    break;
                case "es":
                    sSQL = "select A.job_id, A.date_time, A.job_status_id, B.name_sp, A.assigned_id, A.manifest_num";
                    sSQL += ", A.container_num, A.delivery_address_line1, A.delivery_address_line2, A.delivery_city";
                    sSQL += ", A.delivery_state, A.delivery_postal_code, A.pickup_address_line1, A.pickup_address_line2";
                    sSQL += ", A.pickup_city, A.pickup_state, A.pickup_postal_code, A.username";
                    sSQL += ", A.importer_id, A.importer_name, A.job_num, A.pickup_date_time, A.delivery_date_time";
                    sSQL += ", A.ref_note from jobs as A join job_status_type as B on A.job_status_id = B.id order by A.date_time desc";
                    break;
                default:
                    sSQL = "select A.job_id, A.date_time, A.job_status_id, B.name_en, A.assigned_id, A.manifest_num";
                    sSQL += ", A.container_num, A.delivery_address_line1, A.delivery_address_line2, A.delivery_city";
                    sSQL += ", A.delivery_state, A.delivery_postal_code, A.pickup_address_line1, A.pickup_address_line2";
                    sSQL += ", A.pickup_city, A.pickup_state, A.pickup_postal_code, A.username";
                    sSQL += ", A.importer_id, A.importer_name, A.job_num, A.pickup_date_time, A.delivery_date_time";
                    sSQL += ", A.ref_note from jobs as A join job_status_type as B on A.job_status_id = B.id order by A.date_time desc";
            }
            jobArray = oDB.selectRecordsFromDBList(sSQL, null);
            Log.d("sSQL", sSQL);
        } catch (IOException ioe) {
            DBLogs.insert("Database Exception " + ioe.getMessage(),
                    "JobsCenter.getJobsFromDB",
                    getApplicationContext(), sDBName, iVersionVal);
        } finally {
            oDB.close();
        }

        ListView jobs = (ListView) this
                .findViewById(R.id.lvOpenJobsList);

        ArrayList<JobRow> allJobs = new ArrayList<>();
        Integer iSize = jobArray.size();
        Log.d("iSize", iSize.toString());
        DateConvertClass oDateConvertClass = new DateConvertClass();
        oDateConvertClass.oContext = getApplicationContext();
        if (iSize > 0) {
            for (ArrayList<String> sValue : jobArray) {
                JobRow jobObj = new JobRow();
                jobObj.setJobID(sValue.get(20));
                //Convert the GMT database time to local device time
                jobObj.setJobDate(oDateConvertClass.GMTToLocal(sValue.get(21)));
                jobObj.setJobStatusID(Integer.parseInt(sValue.get(2)));
                jobObj.setJobStatus(sValue.get(3));
                jobObj.setJobManifest(sValue.get(5));
                jobObj.setJobContainer(sValue.get(6));
                allJobs.add(jobObj);
            }

            jobs.setAdapter(new jobRowArrayAdapter(this, 0, allJobs));
            jobs.setOnItemClickListener(new ListView.OnItemClickListener() {
                public void onItemClick(AdapterView<?> parent, View view,
                                        int position, long id) {
                    try {
                        // get the selected row's ticket number
                        Integer iCounter = 0;
                        for (ArrayList<String> sValue : jobArray) {
                            if (position == iCounter) {
                                Log.d("SelectedJobID", sValue.get(0));
                                selectJobs(sValue.get(0));
                            }
                            iCounter++;
                        }
                    } catch (Exception e) {
                        Log.d("Error:", e.toString());
                    }
                }
            });

        } else {
            // No open tickets found
            Log.d("getJobs", "No Messages");
            String[] openTicketsArray = new String[]{getResources().getString(R.string.main_activity_no_jobs)};
            jobs.setAdapter(new ArrayAdapter<String>(this,
                    android.R.layout.simple_list_item_1, openTicketsArray));
        }

    }
    /********************************************************************************/
    /* Handle the selection of a message */

    /**
     * ****************************************************************************
     */
    private void selectJobs(final String sJobID) {
        // call the jobs_detail_activity and pass in the selected job_id
        Intent i = new Intent(getApplicationContext(),
                JobsDetailActivity.class);
        i.putExtra("JobID", sJobID);
        startActivity(i);
        //finish();
    }

    /********************************************************************************/
	/* Handle the selection of a message */

    /**
     * ****************************************************************************
     */
    private void reqJobs() {

        // get the reqJobID
        String sReqJobID = "";
        EditText etReqJobID = (EditText) findViewById(R.id.editJobID);
        sReqJobID = etReqJobID.getText().toString();

        // needs to make sure that there is a reqJobID or do't let them go on
        if (sReqJobID.length() < 1) {
            String sTitle = getResources().getString(R.string.job_center_required_input);
            String sMessage = getResources().getString(R.string.job_center_required_input_message);
            errorDialogue(sTitle, sMessage);
            etReqJobID.setText("");
        } else {

            // need to make sure we don't already have the job number on the device
            if (jobReqAlready()) {
                String sTitle = getResources().getString(R.string.job_center_job_exists);
                String sMessage = getResources().getString(R.string.job_center_job_exists_message);
                errorDialogue(sTitle, sMessage);
                etReqJobID.setText("");
            } else {
                String data = "";
                DBParameters oDBParameters = new DBParameters();
                etReqJobID.setText("");

                // WebServer Request URL
                String serverURL = oDBParameters.readParameter(
                        "REST_BASE_URL", getApplicationContext(), sDBName,
                        iVersionVal); //"http://androidexample.com/media/webservice/JsonReturn.php";

                serverURL += "jobReqSync.php";


                try {
                    // Set Request parameter
                    data += "&" + URLEncoder.encode("reqJobID", "UTF-8") + "=" + sReqJobID;
                    data += "&" + URLEncoder.encode("reqUserID", "UTF-8") + "=" + sUserID;
                } catch (UnsupportedEncodingException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }

                /************ Make Post Call To Web Server ***********/
                BufferedReader reader = null;
                StringBuilder sb = new StringBuilder();

                // Send data
                try {

                    // Defined URL  where to send data
                    URL url = new URL(serverURL);

                    // Send POST data request
                    URLConnection conn = url.openConnection();
                    conn.setDoOutput(true);
                    OutputStreamWriter wr = new OutputStreamWriter(conn.getOutputStream());
                    wr.write(data);
                    wr.flush();

                    // Get the server response
                    reader = new BufferedReader(new InputStreamReader(conn.getInputStream()));
                    String line = null;

                    // Read Server Response
                    while ((line = reader.readLine()) != null) {
                        // Append server response in string
                        sb.append(line + "");
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                    Log.e("SyncJobs Error:", ex.toString());
                } finally {
                    try {
                        reader.close();
                    } catch (Exception ex) {
                    }
                }

                if (sb.length() < 5) {
                    String sTitle = getResources().getString(R.string.job_center_requested_job_does_not_exist);
                    String sMessage = getResources().getString(R.string.job_center_requested_job_does_not_exist_message_a);
                    sMessage += " " + sReqJobID + " ";
                    sMessage += getResources().getString(R.string.job_center_requested_job_does_not_exist_message_b);
                    errorDialogue(sTitle, sMessage);
                }

                // put the information into the database
                TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName, null, iVersionVal);
                try {
                    oDB.createDataBase();
                    oDB.openDataBase();

                    JSONArray oJsonArray = new JSONArray(sb.toString());

                    for (int iCounter = 0; iCounter < oJsonArray.length(); iCounter++) {

                        JSONObject json = oJsonArray.getJSONObject(iCounter);
                        ContentValues oValues = new ContentValues();
                        oValues.put("job_id", json.getString("job_id"));
                        oValues.put("job_status_id", json.getString("job_status_id"));
                        oValues.put("date_time", json.getString("date_time"));
                        oValues.put("status_name", json.getString("status_name"));
                        oValues.put("assigned_id", json.getString("assigned_id"));
                        oValues.put("manifest_num", json.getString("manifest_num"));
                        oValues.put("container_num", json.getString("container_num"));
                        oValues.put("delivery_address_line1", json.getString("delivery_address_line1"));
                        oValues.put("delivery_address_line2", json.getString("delivery_address_line2"));
                        oValues.put("delivery_city", json.getString("delivery_city"));
                        oValues.put("delivery_state", json.getString("delivery_state"));
                        oValues.put("delivery_postal_code", json.getString("delivery_postal_code"));
                        oValues.put("delivery_phone", json.getString("delivery_phone"));
                        oValues.put("delivery_email_address", json.getString("delivery_email_address"));
                        oValues.put("delivery_signature_name", json.getString("delivery_signature_name"));
                        oValues.put("pickup_address_line1", json.getString("pickup_address_line1"));
                        oValues.put("pickup_address_line2", json.getString("pickup_address_line2"));
                        oValues.put("pickup_city", json.getString("pickup_city"));
                        oValues.put("pickup_state", json.getString("pickup_state"));
                        oValues.put("pickup_postal_code", json.getString("pickup_postal_code"));
                        oValues.put("pickup_phone", json.getString("pickup_phone"));
                        oValues.put("pickup_email_address", json.getString("pickup_email_address"));
                        oValues.put("pickup_signature_name", json.getString("pickup_signature_name"));
                        oValues.put("username", json.getString("username"));
                        oValues.put("importer_id", json.getString("importer_id"));
                        oValues.put("importer_name", json.getString("importer_name"));
                        oValues.put("job_num", json.getString("job_num"));
                        oValues.put("entry_number", json.getString("entry_number"));
                        oValues.put("pickup_order_doc_name", json.getString("pickup_order_doc_name"));
                        oValues.put("pickup_date_time", json.getString("pickup_date_time"));
                        oValues.put("delivery_date_time", json.getString("delivery_date_time"));
                        oValues.put("ref_note", json.getString("ref_note"));
                        oValues.put("cargo_weight", json.getString("cargo_weight"));
                        oValues.put("measurement_type_id", json.getString("measurement_type_id"));
                        oValues.put("pickup_signature_name", json.getString("pickup_signature_name"));
                        oValues.put("delivery_signature_name", json.getString("delivery_signature_name"));
                        oDB.insertRecordsInDB("jobs", null, oValues);
                    }
                } catch (Exception ex) {
                    Log.d("getJobs():", ex.toString());
                    DBLogs.insert("Database Exception " + ex.getMessage(),
                            "JobsCenter.REQJobs",
                            getApplicationContext(), sDBName, iVersionVal);
                } finally {
                    oDB.close();
                }
            }
        }
    }

    private boolean jobReqAlready() {
        boolean bRtnVal = false;
        TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName,
                null, iVersionVal);
        try {
            oDB.createDataBase();
        } catch (IOException ioe) {
            throw new Error("Unable to create database");
        }

        EditText etReqJobID = (EditText) findViewById(R.id.editJobID);
        String sReqJobID = etReqJobID.getText().toString();

        oDB.openDataBase();
        String sSQL = "Select * ";
        sSQL += "from jobs ";
        sSQL += " where job_num = '" + sReqJobID + "' COLLATE NOCASE";

        Log.d("sSQL", sSQL);

        jobArray = oDB.selectRecordsFromDBList(sSQL, null);

        // closes the database
        try {
            oDB.close();
        } catch (Throwable e) {
            e.printStackTrace();
        }

        Integer iSize = jobArray.size();
        Log.d("iSize", iSize.toString());
        if (iSize > 0) {
            for (ArrayList<String> sValue : jobArray) {
                bRtnVal = true;
            }
        }
        return bRtnVal;
    }

    private void updateUI() {
        // get the current messages in the DB and display them in the list view
        getJobsFromDB();
    }

    private void errorDialogue(String sTitle, String sMessage) {
        TextView title = new TextView(this);
        title.setText(sTitle);
        title.setBackgroundColor(Color.RED);
        title.setPadding(10, 10, 10, 10);
        title.setGravity(Gravity.CENTER);
        title.setTextColor(Color.WHITE);
        title.setTextSize(20);
        new AlertDialog.Builder(this)
                .setCustomTitle(title)
                .setMessage(sMessage)
                .setPositiveButton(getResources().getString(R.string.ok), new OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        // Do nothing but close the dialog
                        dialog.dismiss();
                    }
                })
                .show();
    }
}
