package com.exquisiteapplicationsolutions.tis.syncClasses;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothManager;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.drawable.Drawable;
import android.location.Location;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.util.Log;

import com.exquisiteapplicationsolutions.tis.DBLogs;
import com.exquisiteapplicationsolutions.tis.DBParameters;
import com.exquisiteapplicationsolutions.tis.DateConvertClass;
import com.exquisiteapplicationsolutions.tis.JobClass;
import com.exquisiteapplicationsolutions.tis.R;
import com.exquisiteapplicationsolutions.tis.TISDBHelper;
import com.exquisiteapplicationsolutions.tis.activities.JobsCenter;
import com.exquisiteapplicationsolutions.tis.objects.Jobs;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.location.LocationListener;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationServices;

import org.json.JSONObject;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Set;
/**
 * Created by John Nissley on 1/16/2017.
 */

public class JobTrackingGPS extends Service implements GoogleApiClient.ConnectionCallbacks,
        GoogleApiClient.OnConnectionFailedListener,
        LocationListener{
    //Debugging
    private static final String TAG = "JobTrackingGPS";
    private Context oContext;
    private BluetoothAdapter myBluetoothAdapter;
    private BlueToothConnection btConnection;
    private BluetoothDevice btDevice = null;
    private static int iVersionVal = 1;
    private static final String sDBName = "TIS.db";
    private static Integer GPS_INSERT_SEC = 60;  // 1 minute
    private static Integer GPS_UPLOAD_SEC = 300;  // 5 minutes
    private Date oTimeofLastPollInsert = new Date(); //This is when we insert a gps value into database
    private Date oTimeofLastGPSUpload = new Date(); //This is when we send GPS data to server
    private Location oLastLocation = null;
    private GoogleApiClient googleLocationClient;
    private ArrayList<Jobs> jobs;
    private ArrayList<ArrayList<String>> GEOFenceArray;
    private NotificationManager notificationManager;

    @Override
    public void onCreate() {

    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        //Bluetooth adapter mac should be sent in the intent
        oContext = getApplicationContext();
        final BluetoothManager btManager = (BluetoothManager)getSystemService(Context.BLUETOOTH_SERVICE);
        myBluetoothAdapter = btManager.getAdapter();
        if (myBluetoothAdapter == null || !myBluetoothAdapter.isEnabled()) {
            //We have a problem.  No bluetooth adapter in device. Need to abort.
            //Should never get here
        }
        btConnection = new BlueToothConnection(getApplicationContext(), BTHandler);
        btDevice = myBluetoothAdapter.getRemoteDevice(findGPSbtMAC());
        doDatabaseData(); //Get the geofence and job information
        // need to get the number of minutes for the sync delay and gps poll mins
        DBParameters oDBParameters = new DBParameters();
        notificationManager = (NotificationManager) oContext.getSystemService(oContext.NOTIFICATION_SERVICE);

        try {
            String sGPSUploadSec = oDBParameters.readParameter("SYNC_GPS_DELAY_SEC",
                    getApplicationContext(), sDBName, iVersionVal);
            String sGPSInsertSec = oDBParameters.readParameter("GPS_POLL_SEC",
                    getApplicationContext(), sDBName, iVersionVal);
            //This is the seconds between uploading the GPS data
            GPS_UPLOAD_SEC = (tryParseInt(sGPSUploadSec, GPS_UPLOAD_SEC));
            //This is the seconds between actual gps gather that will then be uploaded to database
            GPS_INSERT_SEC = (tryParseInt(sGPSInsertSec, GPS_INSERT_SEC));
        } catch (Exception e) {
            DBLogs.insert("Error: " + e.toString(),
                    "JobTrackingGPS._startService()",
                    getApplicationContext(), sDBName, iVersionVal);
        }

        try {
            if (googleLocationClient == null) {
                googleLocationClient = new GoogleApiClient.Builder(this)
                        .addApi(LocationServices.API)
                        .addConnectionCallbacks(this)
                        .addOnConnectionFailedListener(this)
                        .build();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        btConnection.connect(btDevice);
        return Service.START_STICKY_COMPATIBILITY;
    }

    /** Called when The service is no longer needed and is being destroyed */
    @Override
    public void onDestroy() {
        super.onDestroy();
        //Need to close the bluetooth connection
        //Do not send anything to the gps unit since this may have been caused by the os
        //so we will want the unit to store gps data until the process starts again
        //NotificationManager notificationManager = (NotificationManager)getSystemService(Context.NOTIFICATION_SERVICE);
        notificationManager.cancel(90);
        String doNotSaveData = "@14|10\n";
        btConnection.write(doNotSaveData.getBytes());
        if (googleLocationClient != null && googleLocationClient.isConnected()) {
            googleLocationClient.disconnect();
        }
        btConnection.stop();
        BTHandler.removeCallbacksAndMessages(null);
        //We must send any remaining coords to the server
        SynJobTracking.oContext = getApplicationContext();
        SynJobTracking.sendJobCoords();
        //We need to clear out the gps coords so the will not be there if we start again
        SharedPreferences enteredFences = getApplicationContext().getSharedPreferences("geofenceInfo", Context.MODE_PRIVATE);
        enteredFences.edit().clear().apply();
    }

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    /**
     * The Handler that gets information back from the BluetoothChatService
     */
    private final Handler BTHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case BlueToothConstants.MESSAGE_STATE_CHANGE:
                    switch (msg.arg1) {
                        case BlueToothConnection.STATE_CONNECTED:
                            Log.e(TAG, "State: Connected");
                            //Keep data if we get disconnected
                            String saveData = "@14|11\n";
                            btConnection.write(saveData.getBytes());
                            int longUpdate = GPS_INSERT_SEC;
                            String pollTiming = "@13|2|" + longUpdate + "\n";
                            btConnection.write(pollTiming.getBytes());
                            if (googleLocationClient != null && googleLocationClient.isConnected()) {
                                googleLocationClient.disconnect();
                            }
                            createNotification();
                            break;
                        case BlueToothConnection.STATE_CONNECTING:
                            Log.e(TAG, "State: Connecting");
                            break;
                        case BlueToothConnection.STATE_NONE:
                            Log.e(TAG, "State: None");
                            //if (googleLocationClient != null && !googleLocationClient.isConnected()) {
                            //    googleLocationClient.connect();
                            //}
                            break;
                        case BlueToothConnection.STATE_LOST_CONNECTION:
                            Log.e(TAG, "State: Lost Connection");
                            new Handler().postDelayed(new Runnable() {
                                @Override
                                public void run() {
                                    btConnection.connect(btDevice);
                                }
                            }, 15000);
                            if (googleLocationClient != null && !googleLocationClient.isConnected()) {
                                googleLocationClient.connect();
                            }
                            //notificationManager = (NotificationManager)getSystemService(Context.NOTIFICATION_SERVICE);
                            notificationManager.cancel(90);
                            break;
                        case BlueToothConnection.STATE_CONNECTION_FAILED:
                            Log.e(TAG, "State: Connection Failed");
                            new Handler().postDelayed(new Runnable() {
                                @Override
                                public void run() {
                                    btConnection.connect(btDevice);
                                }
                            }, 15000);
                            if (googleLocationClient != null && !googleLocationClient.isConnected()) {
                                googleLocationClient.connect();
                            }
                            //notificationManager = (NotificationManager)getSystemService(Context.NOTIFICATION_SERVICE);
                            notificationManager.cancel(90);
                            break;
                    }
                    break;
                case BlueToothConstants.MESSAGE_READ:
                    // construct a string from the valid bytes in the buffer
                    String readMessage = msg.obj.toString();
                    Log.e(TAG, "State: Message Read");
                    Log.e(TAG, readMessage);
                    parseBTInput(readMessage.split("\\|"));
                    break;
            }
        }
    };

    private void createNotification () {
        int iJob = 0;
        if (getStatusBarBackgroundLightValue() < 128) {
            iJob = R.drawable.truck_bt_menu;
        } else {
            iJob = R.drawable.truck_bt;
        }
        Intent notifyIntent = new Intent(oContext.getApplicationContext(), JobsCenter.class);
        PendingIntent resultPendingIntent =
                PendingIntent.getActivity(
                        oContext.getApplicationContext(),
                        0,
                        notifyIntent,
                        PendingIntent.FLAG_UPDATE_CURRENT
                );
        String sTitle = getResources().getString(R.string.bluetooth_gps_title);
        String sMessage = getResources().getString(R.string.bluetooth_gps_message);
        Notification noti = new Notification.Builder(oContext)
                .setContentTitle(sTitle)
                .setContentText(sMessage)
                .setContentIntent(resultPendingIntent)
                .setSmallIcon(iJob).build();
        //notificationManager = (NotificationManager) oContext.getSystemService(oContext.NOTIFICATION_SERVICE);
        // hide the notification after its selected
        noti.flags |= Notification.FLAG_NO_CLEAR;  //Do not allow user to clear notification
        notificationManager.notify(90, noti);
    }

    /*
    * Returns estimated value of the lightness of the status bar background
    * @return
            */
    private int getStatusBarBackgroundLightValue()
    {
        // better this than nothing
        Drawable bg = oContext.getResources().getDrawable(android.R.drawable.status_bar_item_background);
        int height = Math.max(1, bg.getIntrinsicHeight());
        int width = Math.max(1, bg.getIntrinsicWidth());
        Bitmap bitmap = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888);
        Canvas canvas = new Canvas(bitmap);
        bg.setBounds(0, 0, width, height);
        bg.draw(canvas);

        long sum = 0;
        for (int x=0; x<width; x++){
            for (int y=0; y<height; y++){
                int color = bitmap.getPixel(x, y);
                int r = (color >> 16) & 0xFF;
                int g = (color >> 8) & 0xFF;
                int b = (color) & 0xFF;
                int max = Math.max(r, Math.max(g, b));
                int min = Math.min(r, Math.min(g, b));
                int l = (min + max)/2;
                sum = sum + l;
            }
        }
        bitmap.recycle();
        bitmap = null;
        canvas = null;
        bg = null;
        sum = sum / (width * height);
        // should be [0..255]
        return (int)Math.min(255, Math.max(sum, 0));
    }

    private void doDatabaseData () {
        JobClass jobClass = new JobClass();
        TISDBHelper oDB = new TISDBHelper(oContext, sDBName,
                null, iVersionVal);
        try {
            oDB.createDataBase();
            oDB.openDataBase();
            String sSQL = "select name, lat, long, isEntry, isExit, gpsGroupName, radius, isInPort from gps_coords " +
                    "where isGeofence = 1";
            Log.d("sSQL", sSQL);
            //Update the geofence variable
            GEOFenceArray = oDB.selectRecordsFromDBList(sSQL, null);
            //Read all jobs
            jobs = jobClass.read(oContext);
        } catch (IOException ioe) {
            throw new Error("Unable to create database");
        } finally {
            oDB.close();
        }
    }

    private void doDatabaseData (Location oLocation) {
        JobClass jobClass = new JobClass();
        //Reads all open jobs
        TISDBHelper oDB = new TISDBHelper(oContext, sDBName,
                null, iVersionVal);
        try {
            oDB.createDataBase();
            oDB.openDataBase();
            String sSQL = "select name, lat, long, isEntry, isExit, gpsGroupName, radius, isInPort from gps_coords " +
                    "where isGeofence = 1";
            Log.d("sSQL", sSQL);
            //Update the geofence variable
            //GEOFenceArray = oDB.selectRecordsFromDBList(sSQL, null);
            jobs = jobClass.read(oContext);
            float fDistance = 0;
            if (oLastLocation != null) {
                fDistance = oLocation.distanceTo(oLastLocation);
            }
            //Need to iterate through one job at a time
            if (!jobs.isEmpty()) {
                for (Jobs job : jobs) {
                    ContentValues oValues = SynJobTracking.prepJobDatabaseData(oLocation, job, fDistance);
                    oDB.insertRecordsInDB("gpsHistory", null, oValues);
                }
            }
        } catch (IOException ioe) {
            throw new Error("Unable to create database");
        } finally {
            oDB.close();
        }
    }

    private void doUploadGPSData () {
        //We do this within a thread since it is time consuming and we do not want to
        //slow down the rest of the service since it is processing data often
        new Thread(new Runnable() {
            @Override
            public void run() {
                JSONObject oReturnedValues = SynJobTracking.sendJobCoords();
                try {
                    if (oReturnedValues.getString("success").toLowerCase().equals("true")) {
                        Integer iAdjustSeconds = Integer.parseInt(oReturnedValues.getString("adjust_seconds"));
                        Calendar cal = Calendar.getInstance();
                        cal.setTime(oTimeofLastGPSUpload);
                        cal.add(Calendar.SECOND, iAdjustSeconds);
                        oTimeofLastGPSUpload = cal.getTime();
                    }
                } catch(Exception e) {

                }
            }
        }).start();
    }

    private String findGPSbtMAC() {
        String macAddress = null;
        // Get a set of currently paired devices
        Set<BluetoothDevice> pairedDevices = myBluetoothAdapter.getBondedDevices();
        // If there are paired devices, add each one to the ArrayAdapter
        if (pairedDevices.size() > 0) {
            for (BluetoothDevice device : pairedDevices) {
                if (device.getName().toLowerCase().equals("tis-gps")) {
                    macAddress = device.getAddress();
                    break;
                }
            }
        }
        return macAddress;
    }

    private void parseBTInput(String[] items) {
        Integer apiCode = tryParseInt(items[0], 0);
        switch (apiCode){
            case 10:
                if (items.length > 8) {
                    //This is live gps data
                    Location oLocation = parseGPSData(items);
                    String sReturnMessage = "@12|" + items[1] + "|" + items[2] + "\n";
                    Log.d(TAG, sReturnMessage);
                    btConnection.write(sReturnMessage.getBytes());
                    checkGeoFence(oLocation, Boolean.TRUE);
                }
                break;
            case 11:
                if (items.length > 8) {
                    //This is delayed gps data
                    Location oLocation = parseGPSData(items);
                    String sReturnMessage = "@12|" + items[1] + "|" + items[2] + "\n";
                    Log.d(TAG, sReturnMessage);
                    getJobDates(oLocation);
                    btConnection.write(sReturnMessage.getBytes());
                }
                break;
        }
    }

    private Location parseGPSData(String[] items) {
        long i = 0;
        Location oReturnLocation = new Location(items[0]);
        DateConvertClass oDateEpoch = new DateConvertClass();
        oDateEpoch.oContext = oContext;
        oReturnLocation.setTime(oDateEpoch.GPSStringToEpoch(items[1], items[2]));
        oReturnLocation.setLatitude(tryParseDouble(items[3]));
        oReturnLocation.setLongitude(tryParseDouble(items[4]));
        oReturnLocation.setSpeed(tryParseLong(items[5], i));
        oReturnLocation.setAltitude(tryParseLong(items[6], i));
        oReturnLocation.setAccuracy(tryParseFloat(items[7]));
        return oReturnLocation;
    }

    private void checkGeoFence(Location oCurrentLocation, Boolean updateDatabase) {
        //All live GPS data comes through here so we to all of the time related stuff here
        if (oCurrentLocation.getLatitude() != 0 && oCurrentLocation.getLongitude() != 0) {
            Date oNow = new Date();
            long lSaveGPSLocalDiff = oNow.getTime() - oTimeofLastPollInsert.getTime();
            long lUploadGPSDiff = oNow.getTime() - oTimeofLastGPSUpload.getTime();
            Integer iSaveGPSLocalDiffSeconds = (int) lSaveGPSLocalDiff / 1000;
            Integer iUploadGPSDiffSeconds = (int) lUploadGPSDiff / 1000;
            //Log.d(TAG, "lSave=" + Long.valueOf(iSaveGPSLocalDiffSeconds) + " lUpload=" + Long.valueOf(iUploadGPSDiffSeconds) + " Insert Sec=" + Long.valueOf(GPS_INSERT_SEC) + " Upload Sec=" + Long.valueOf(GPS_UPLOAD_SEC));
            SynJobTracking.oContext = getApplicationContext();
            boolean bCrossGEOFence = false;
            if (GEOFenceArray.size() > 0) {
                bCrossGEOFence = SynJobTracking.testForGEOFence(jobs, oCurrentLocation, GEOFenceArray);
            }
            if (updateDatabase) {
                if (bCrossGEOFence || iSaveGPSLocalDiffSeconds >= GPS_INSERT_SEC) {
                    doDatabaseData(oCurrentLocation);
                    oTimeofLastPollInsert = oNow;
                }
                if (iUploadGPSDiffSeconds >= GPS_UPLOAD_SEC) {
                    doUploadGPSData();
                    oTimeofLastGPSUpload = oNow;
                }
            }
        }
    }

    // Get all jobs and compare the start and end dates to the delayed gps data to see if
    // we need to update the server database with gps data that may have happened when
    // disconnected from the bluetooth device.
    private void getJobDates(final Location oLocation) {
        new Thread(new Runnable() {
            @Override
            public void run() {
                final Location oCurrentLocation = oLocation;
                JobClass oJobClass = new JobClass();
                jobs = oJobClass.getAllJobs(oContext);
                TISDBHelper oDB = new TISDBHelper(oContext, sDBName,
                        null, iVersionVal);
                try {
                    oDB.createDataBase();
                    oDB.openDataBase();
                    //Need to iterate through one job at a time
                    DateConvertClass oDateConvertClass = new DateConvertClass();
                    String gpsDate = oDateConvertClass.EpochToGMTTimestamp(oCurrentLocation.getTime());
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                    Date oGPSDate = null;
                    try {
                        oGPSDate = sdf.parse(gpsDate);
                    } catch(Exception ex) {
                        Log.d(TAG, "GPSDate = " + oGPSDate.toString());
                    }
                    //Need to iterate through one job at a time
                    if (!jobs.isEmpty()) {
                        for (Jobs job : jobs) {
                            //Check if gps date is between job open and close date
                            Boolean bWriteDate = false;
                            try{
                                if (!job.start_date.isEmpty()) {
                                    Date oStartDate = sdf.parse(job.start_date);
                                    if (oStartDate.before(oGPSDate)) {
                                        bWriteDate = true;
                                    }
                                    Log.d(TAG, "Job Start Date = " + oStartDate.toString());
                                }
                                if (!job.stop_date.isEmpty()) {
                                    Date oStopDate = sdf.parse(job.stop_date);
                                    if (oStopDate.before(oGPSDate)) {
                                        bWriteDate = false;
                                    }
                                    Log.d(TAG, "Job Stop Date = " + oStopDate.toString());
                                }

                            }catch(Exception ex) {
                                Log.d(TAG, "Failed date conversion.");
                            }
                            if (bWriteDate ) {
                                ContentValues oValues = SynJobTracking.prepJobDatabaseData(oCurrentLocation, job, (float) 0.0);
                                Log.d(TAG, "Adding to Job ID = " + job.job_id);
                                oDB.insertRecordsInDB("gpsHistory", null, oValues);
                            }
                        }
                    }
                } catch (IOException ioe) {
                    throw new Error("Unable to create database");
                } finally {
                    oDB.close();
                }
            }
        }).start();
    }

    private Integer tryParseInt(String text, Integer defaultValue) {
        try {
            return Integer.parseInt(text.trim());
        } catch (NumberFormatException e) {
            return defaultValue;
        }
    }

    private Double tryParseDouble(String text) {
        try {
            return Double.parseDouble(text.trim());
        } catch (NumberFormatException e) {
            return 0.0;
        }
    }

    private Long tryParseLong(String text, Long defaultValue) {
        try {
            return Long.parseLong(text.trim());
        } catch (NumberFormatException e) {
            return defaultValue;
        }
    }

    private Float tryParseFloat(String text) {
        try {
            return Float.parseFloat(text.trim());
        } catch (NumberFormatException e) {
            return Float.parseFloat("0");
        }
    }

    @Override
    public void onLocationChanged(Location location) {
        final Location oLocation = location;
        String test = String.valueOf(oLocation.getLatitude());
        Log.d(TAG, "Lat = " + test + " " + oLocation.getTime());
        checkGeoFence(location, Boolean.FALSE);
    }

    //GoogleApiClient
    @Override
    public void onConnectionFailed(ConnectionResult bundle) {
        Log.d(TAG, "Google services connection failed");
    }

    @Override
    public void onConnected(Bundle bundle) {
        Log.d(TAG, "Google services connected");
        LocationRequest locationRequest = new LocationRequest();
        locationRequest.setInterval(10000); // 10 seconds
        locationRequest.setFastestInterval(3000); // 3 seconds
        locationRequest.setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY);
        LocationServices.FusedLocationApi.requestLocationUpdates(
                googleLocationClient, locationRequest, this);
    }

    @Override
    public void onConnectionSuspended(int i) {

    }
}
