package com.exquisiteapplicationsolutions.tis.syncClasses;

import android.app.Notification;
import android.app.NotificationManager;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.drawable.Drawable;
import android.location.Location;
import android.util.Log;

import com.exquisiteapplicationsolutions.tis.DBLogs;
import com.exquisiteapplicationsolutions.tis.DateConvertClass;
import com.exquisiteapplicationsolutions.tis.R;
import com.exquisiteapplicationsolutions.tis.TISDBHelper;
import com.exquisiteapplicationsolutions.tis.objects.Jobs;
import com.exquisiteapplicationsolutions.tis.webServices.SyncJson;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;

/**
 * Created by jcairns on 6/19/15.
 */
public class SynJobTracking {
    private static final String sDBName = "TIS.db";
    public static Context oContext;
    private static final int iVersionVal = 1;

    public static JSONObject sendJobCoords() {
        JSONObject oReturnValue = new JSONObject();
        try {
            oReturnValue.put("success", "false");
            oReturnValue.put("adjust_seconds", "0");
        } catch(Exception e) {
            DBLogs.insert("Unable to init json object",
                    "SynJobTracking.sendJobCoords",
                    oContext, sDBName, iVersionVal);
        }
        String sJobNote = "Automated updated from the Android device with current GPS information.";
        ArrayList<ArrayList<String>> gpsArray = new ArrayList<>();

        TISDBHelper oDB = new TISDBHelper(oContext, sDBName,
                null, iVersionVal);
        try {
            oDB.createDataBase();
            oDB.openDataBase();
            String sSql = "Select userID, JobID, Lat, Long, Time, Accuracy, Speed, Distance, Type from gpsHistory";
            Log.d("sSQL", sSql);
            gpsArray = oDB.selectRecordsFromDBList(sSql, null);
            oDB.deleteAllRecords("gpsHistory");
        } catch (Exception ex) {
            Log.d("sendJobCoords Exception", ex.toString());
            //Todo
        } finally {
            oDB.close();
        }
        if (gpsArray.size() > 0) {
            JSONArray gpsItemArray = new JSONArray();
            for (ArrayList<String> sValue : gpsArray) {
                JSONObject gpsItem = new JSONObject();
                try {
                    gpsItem.put("userID", sValue.get(0));
                    gpsItem.put("job_id", sValue.get(1));
                    gpsItem.put("jobStatus_id", "4");
                    gpsItem.put("notes", sJobNote);
                    gpsItem.put("loc_id1", sValue.get(2));
                    gpsItem.put("loc_id2", sValue.get(3));
                    gpsItem.put("datetime", sValue.get(4));
                    gpsItem.put("accuracy", sValue.get(5));
                    gpsItem.put("speed", sValue.get(6));
                    gpsItem.put("distance", sValue.get(7));
                    gpsItem.put("type", sValue.get(8));
                    //Place json item into array that will be passed to webservice
                    gpsItemArray.put(gpsItem);
                } catch (Exception ex) {
                    Log.d("json exception", ex.toString());
                }
            }
            //send it to the web service
            SyncJson oSyncgpsItems = new SyncJson();
            oSyncgpsItems.setType(SyncJson.Types.gps);
            oSyncgpsItems.setContext(oContext);
            oSyncgpsItems.setSendString(gpsItemArray.toString());
            oSyncgpsItems.setWebserviceName("insJobTracking01WS.php");
            oSyncgpsItems.SyncData();
            if (!oSyncgpsItems.getSuccess()) {
                //We must put all of the data back into the local database since it
                //did not get loaded to the server
                DBLogs.insert("sendjobcoords was not successful",
                        "Send Job Coords not successful",
                        oContext, sDBName, iVersionVal);
                try {
                    oDB.createDataBase();
                    oDB.openDataBase();
                    if (gpsArray.size() > 0) {
                        for (ArrayList<String> sValue : gpsArray) {
                            ContentValues oValues = new ContentValues();
                            oValues.put("userID", sValue.get(0));
                            oValues.put("JobID", sValue.get(1));
                            oValues.put("Lat", sValue.get(4));
                            oValues.put("Long", sValue.get(5));
                            oValues.put("Time", sValue.get(6));
                            oValues.put("Accuracy", sValue.get(7));
                            oValues.put("Speed", sValue.get(8));
                            oValues.put("Distance", sValue.get(9));
                            oValues.put("Type", sValue.get(10));
                            oDB.insertRecordsInDB("gpsHistory", null, oValues);
                        }
                    }
                } catch (Exception ex) {
                    Log.d("sendJobCoords Exception", ex.toString());
                    //Todo
                } finally {
                    oDB.close();
                }
            }

        }
        return oReturnValue;
    }

    /*
    We only get here if there is an active job. Test where the driver is related to the
    geofences that have been set up
     */

    public static boolean testForGEOFence(ArrayList<Jobs> jobs, Location location, ArrayList<ArrayList<String>> GEOFenceArray) {
        boolean bRtnValue = false;
        if (GEOFenceArray.size() > 0) {
            int iFence;
            if (getStatusBarBackgroundLightValue() < 128) {
                iFence = R.drawable.geo_fence_menu;
            } else {
                iFence = R.drawable.geo_fence;
            }

            Integer iNotificationNum = 10;
            for (ArrayList<String> sValue : GEOFenceArray) {
                Boolean bNotify = false;
                Double dGEOFenceLat = tryParseDouble(sValue.get(1));
                Double dGEOFenceLong = tryParseDouble(sValue.get(2));
                Integer iRadius = tryParseInt(sValue.get(6), 50);
                String sFenceName = sValue.get(0).toString();
                String sFenceGroupName = sValue.get(5).toString();
                Integer iIsEntry = Integer.parseInt(sValue.get(3));
                Integer iIsExit = Integer.parseInt(sValue.get(4));
                Integer isInPort = Integer.parseInt(sValue.get(7));
                Location lFenceLocation = new Location("Fence");
                String sNotiText = "";
                lFenceLocation.setLatitude(dGEOFenceLat);
                lFenceLocation.setLongitude(dGEOFenceLong);
                Boolean bPartOfGroup = false;
                float fFenceDistance = location.distanceTo(lFenceLocation);
                //Make sure to iterate through each job
                for (Jobs jobsClass : jobs) {
                    Integer iFenceStatus = fenceStatus(jobsClass.job_id + "-" + sFenceName);
                    Integer iFenceGroupStatus = -1; //Set as no group fence status exists
                    if (!sFenceGroupName.equals("null")) {
                        iFenceGroupStatus = fenceStatus(jobsClass.job_id + "-" + sFenceGroupName);
                        bPartOfGroup = true;
                    }
                    if (fFenceDistance < iRadius) {
                        if (iFenceStatus == 0) {
                            //This means we have never been inside the fence before
                            updateGEOStatus("ENTER", jobsClass.job_id + "-" + sFenceName); //Update the fence status pref file
                            if (iIsEntry == 1) {
                                //This is a fence we will show as an entry
                                bNotify = true;
                                sNotiText = "Entered location " + sFenceName + ".";
                                if (bPartOfGroup && iFenceGroupStatus == 0) {
                                    updateGEOStatus("ENTER", jobsClass.job_id + "-" + sFenceGroupName);
                                    if (isInPort == 1) {
                                        sNotiText = "Entry -  Entered " + sFenceGroupName;
                                        sNotiText += " through " + sFenceName + ".";
                                    } else {
                                        sNotiText = "Port - " + sFenceGroupName;
                                        sNotiText += " through " + sFenceName + ".";
                                    }
                                } else {
                                    if (bPartOfGroup) {
                                        bNotify = false;
                                    }
                                }
                            }
                        }
                    }

                    if (fFenceDistance > iRadius) {
                        if (iFenceStatus == 1) {
                            //This means we were inside of the fence on the last check
                            updateGEOStatus("EXIT", jobsClass.job_id + "-" + sFenceName);
                            if (iIsExit == 1) {
                                //This is a fence we will show as an exit
                                bNotify = true;
                                sNotiText = "Exited location " + sFenceName + ".";
                                //if (!sFenceGroupName.equals("null")) {
                                if (bPartOfGroup && iFenceGroupStatus == 1) {
                                    updateGEOStatus("EXIT", jobsClass.job_id + "-" + sFenceGroupName);
                                    sNotiText = "Exit - Exited " + sFenceGroupName;
                                    sNotiText += " through " + sFenceName + ".";
                                } else {
                                    if (bPartOfGroup) {
                                        bNotify = false;
                                    }
                                }
                            }
                        }
                    }

                    if (bNotify) {
                        //send notification to the server
                        updateJobStatus(jobsClass, sNotiText);
                        //send notification to user on the phone
                        Notification noti = new Notification.Builder(oContext)
                                .setContentTitle("Location")
                                .setContentText(sNotiText)
                                //.setStyle(new Notification.BigTextStyle().bigText(sNotiText))
                                .setSmallIcon(iFence)
                                .build();
                        NotificationManager notificationManager = (NotificationManager)
                                oContext.getSystemService(oContext.NOTIFICATION_SERVICE);
                        notificationManager.notify(iNotificationNum, noti);
                        iNotificationNum++;
                        bRtnValue = true;
                    }
                }
            }
        }
        return bRtnValue;
    }

    public static ContentValues prepJobDatabaseData(Location oLocation, Jobs job, Float fdistance) {
        SharedPreferences prefs = oContext.getSharedPreferences("TISSharedPrefs",
                Context.MODE_PRIVATE);
        String sUserID = prefs.getString("UserID", "No Value");
        ContentValues returnValue = new ContentValues();
        returnValue.put("JobID", job.job_id.toString());
        returnValue.put("Lat", String.valueOf(oLocation.getLatitude()));
        returnValue.put("Long", String.valueOf(oLocation.getLongitude()));
        DateConvertClass sGMTTimeStamp = new DateConvertClass();
        sGMTTimeStamp.oContext = oContext;
        returnValue.put("Time", sGMTTimeStamp.EpochToGMTTimestamp(oLocation.getTime()));
        returnValue.put("userID", sUserID.toString());
        returnValue.put("Accuracy", Float.valueOf(oLocation.getAccuracy()));
        returnValue.put("Speed", Float.valueOf(oLocation.getSpeed()));
        returnValue.put("Distance", "unknown");
        returnValue.put("Type", "Job");
        return returnValue;
    }

    public static void saveLocationDataToDatabase(ArrayList<String> alCoordArray) {
        TISDBHelper oDB = new TISDBHelper(oContext, sDBName,
                null, iVersionVal);
        try {
            oDB.createDataBase();
            oDB.openDataBase();
            for (String coord : alCoordArray) {
                JSONObject oCoord = new JSONObject(coord);
                ContentValues oValues = new ContentValues();
                oValues.put("JobID", oCoord.getString("JobID"));
                oValues.put("Lat", oCoord.getString("Lat"));
                oValues.put("Long", oCoord.getString("Long"));
                oValues.put("Time", oCoord.getString("Time"));
                oValues.put("userID", oCoord.getString("userID"));
                oValues.put("Accuracy", oCoord.getString("Accuracy"));
                oValues.put("Speed", oCoord.getString("Speed"));
                oValues.put("Distance", oCoord.getString("Distance"));
                oValues.put("Type", oCoord.getString("Type"));
                oDB.insertRecordsInDB("gpsHistory", null, oValues);
            }
        } catch(Exception ex) {
            Log.d("Database Error", "Save Location to Database error.");
            //Todo
        } finally {
            oDB.close();
        }
    }

    /*
    Here we are checking the status of the fence
    0 = never entered or exited
    1 = entered but never exited
    2 = entered and exited
    Assumption is that you can not exit if you have not already entered
     */
    private static Integer fenceStatus(String sFenceName) {
        Integer iRtnValue = 0;
        SharedPreferences enteredFences = oContext.getSharedPreferences("geofenceInfo", Context.MODE_PRIVATE);
        String sValueEnter = enteredFences.getString(sFenceName + "-Enter", null);
        String sValueExit = enteredFences.getString(sFenceName + "-Exit", null);
        //if there is a value returned for the enter and not one returned for the exit then return
        //true for inside fence. If we have already exited the fence we can not enter again.
        if (sValueEnter != null && sValueExit == null) {
            iRtnValue = 1;
        }
        if (sValueEnter != null && sValueExit != null) {
            iRtnValue = 2;
        }
        return iRtnValue;
    }

    private static void updateGEOStatus(String sInorOut, String sFenceName) {
        SharedPreferences enteredFences = oContext.getSharedPreferences("geofenceInfo", Context.MODE_PRIVATE);
        if (sInorOut.toUpperCase().equals("ENTER")) {
            enteredFences.edit().putString(sFenceName + "-Enter", getGMTDataTime()).apply();
        }
        if (sInorOut.toUpperCase().equals("EXIT")) {
            enteredFences.edit().putString(sFenceName + "-Exit", getGMTDataTime()).apply();
        }
    }

    private static void updateJobStatus(Jobs JobClass, String sUpdate) {
        try {
            DateConvertClass oDateConvertClass = new DateConvertClass();
            oDateConvertClass.oContext = oContext;
            String sCurrentDateTime = oDateConvertClass.GMTTimestamp();
            JSONObject oDocStatus = new JSONObject();
            oDocStatus.put("jobID", JobClass.job_num.toString());
            oDocStatus.put("userID", JobClass.username.toString());
            oDocStatus.put("PUDelStatus", sUpdate);
            oDocStatus.put("dateTime", sCurrentDateTime);
            // Now set up the sync queue and try to send now
            SyncTasks syncTasks = new SyncTasks();
            syncTasks.oContext = oContext;
            syncTasks.sRecType = "ORDERPU";
            syncTasks.sRecID = oDocStatus.toString();
            syncTasks.sDateTime = sCurrentDateTime;
            syncTasks.sQueued = "1"; //Set as queued so sync service will not pick it up
            syncTasks.sDateTimeQueued = sCurrentDateTime;
            syncTasks.sCompleted = "0";
            syncTasks.sDateTimeCompleted = "";
            long lRecID = syncTasks.write();
            Intent OrderPUIntent = new Intent(oContext, SyncIntentService.class);
            OrderPUIntent.putExtra("RecType", "ORDERPU");
            OrderPUIntent.putExtra("ID", Long.toString(lRecID));
            OrderPUIntent.putExtra("RecID", oDocStatus.toString());
            oContext.startService(OrderPUIntent);
        } catch (JSONException ex) {
            DBLogs.insert("JSON Error: " + ex.toString(),
                    "PickupOrderActivity.acceptDocument", oContext,
                    sDBName, iVersionVal);
        }
    }

    /*
    * Returns estimated value of the lightness of the status bar background
    * @return
            */
    private static int getStatusBarBackgroundLightValue()
    {
        // better this than nothing
        Drawable bg = oContext.getResources().getDrawable(android.R.drawable.status_bar_item_background);
        int height = Math.max(1, bg.getIntrinsicHeight());
        int width = Math.max(1, bg.getIntrinsicWidth());
        Bitmap bitmap = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888);
        Canvas canvas = new Canvas(bitmap);
        bg.setBounds(0, 0, width, height);
        bg.draw(canvas);

        long sum = 0;
        for (int x=0; x<width; x++){
            for (int y=0; y<height; y++){
                int color = bitmap.getPixel(x, y);
                int r = (color >> 16) & 0xFF;
                int g = (color >> 8) & 0xFF;
                int b = (color) & 0xFF;
                int max = Math.max(r, Math.max(g, b));
                int min = Math.min(r, Math.min(g, b));
                int l = (min + max)/2;
                sum = sum + l;
            }
        }
        bitmap.recycle();
        bitmap = null;
        canvas = null;
        bg = null;
        sum = sum / (width * height);
        // should be [0..255]
        return (int)Math.min(255, Math.max(sum, 0));
    }

    // Get the GMT time that will be inserted into any log messages
    private static String getGMTDataTime() {
        DateConvertClass oDateConvertClass = new DateConvertClass();
        oDateConvertClass.oContext = oContext;
        return oDateConvertClass.GMTTimestamp();
    }

    private static Integer tryParseInt(String text, Integer defaultValue) {
        try {
            return Integer.parseInt(text);
        } catch (NumberFormatException e) {
            return defaultValue;
        }
    }

    private static Double tryParseDouble(String text) {
        try {
            return Double.parseDouble(text);
        } catch (NumberFormatException e) {
            return 0.0;
        }
    }

    private static Long tryParseLong(String text, Long defaultValue) {
        try {
            return Long.parseLong(text);
        } catch (NumberFormatException e) {
            return defaultValue;
        }
    }

    private static Float tryParseFloat(String text) {
        try {
            return Float.parseFloat(text);
        } catch (NumberFormatException e) {
            return Float.parseFloat("0");
        }
    }
}
