package com.exquisiteapplicationsolutions.tis.syncClasses;

import android.app.Activity;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.drawable.Drawable;
import android.util.Log;

import com.exquisiteapplicationsolutions.tis.DBLogs;
import com.exquisiteapplicationsolutions.tis.DBParameters;
import com.exquisiteapplicationsolutions.tis.R;
import com.exquisiteapplicationsolutions.tis.TISDBHelper;
import com.exquisiteapplicationsolutions.tis.activities.UserMessages;
import com.exquisiteapplicationsolutions.tis.objects.MessagesClass;

import org.json.JSONArray;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.util.ArrayList;


public class SyncMessages {
    private static final int TIMEOUT_MILLISEC = 600;
    private static final String sDBName = "TIS.db";
    public static Activity inActivity;
    public static Context oContext;
    public static String sUserName;
    static int iVersionVal = 1;

    public static void getMessages() throws Exception {
        String data = "";

        // need to get the last message id that was last syncd from the server
        int iLastSyncMsgID = lastMsgIdSync();

        DBParameters oDBParameters = new DBParameters();

        // WebServer Request URL
        String serverURL = oDBParameters.readParameter(
                "REST_BASE_URL", oContext, sDBName,
                iVersionVal); //"http://androidexample.com/media/webservice/JsonReturn.php";

        serverURL += "messagesSync.php";

        try {
            // Set Request parameter
            data += "&" + URLEncoder.encode("userName", "UTF-8") + "=" + sUserName;
            data += "&" + URLEncoder.encode("lastMsgId", "UTF-8") + "=" + iLastSyncMsgID;
        } catch (UnsupportedEncodingException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        /************ Make Post Call To Web Server ***********/
        BufferedReader reader = null;
        StringBuilder sb = new StringBuilder();

        // Send data
        try {

            // Defined URL  where to send data
            URL url = new URL(serverURL);

            // Send POST data request
            URLConnection conn = url.openConnection();
            conn.setDoOutput(true);
            OutputStreamWriter wr = new OutputStreamWriter(conn.getOutputStream());
            wr.write(data);
            wr.flush();

            // Get the server response
            reader = new BufferedReader(new InputStreamReader(conn.getInputStream()));
            String line = null;

            // Read Server Response
            while ((line = reader.readLine()) != null) {
                // Append server response in string
                sb.append(line + "");

            }
        } catch (Exception ex) {
            ex.printStackTrace();
            Log.e("SyncMessages Error:", ex.toString());
        } finally {
            try {
                reader.close();
            } catch (Exception ex) {
            }
        }

        // put the information into the database
        TISDBHelper oDB = new TISDBHelper(oContext, sDBName, null, iVersionVal);
        try {
            oDB.createDataBase();
        } catch (IOException ioe) {
            throw new Error("Unable to create database");
        }

        oDB.openDataBase();

        Boolean bGotAMessage = false;
        Integer iMessageCount = 0;
        try {
            JSONArray oJsonArray = new JSONArray(sb.toString());

            for (int iCounter = 0; iCounter < oJsonArray.length(); iCounter++) {
                iMessageCount++;
                JSONObject json = oJsonArray.getJSONObject(iCounter);
                ContentValues oValues = new ContentValues();
                oValues.put("ToUserFullName", json.getString("to_user_full_name"));
                oValues.put("MsgID", json.getString("id"));
                oValues.put("FromUserFullName", json.getString("from_user_full_name"));
                oValues.put("ToUserName", json.getString("to_username"));
                oValues.put("FromUserName", json.getString("from_username"));
                oValues.put("MsgDate", json.getString("date_time"));
                oValues.put("Message", json.getString("message"));
                oValues.put("MsgStatus", json.getString("status_name"));

                try {
                    oDB.insertRecordsInDB("Messages", null, oValues);
                    bGotAMessage = true;
                } catch (Exception sqlEx) {
                    Log.e("sqlEx:", sqlEx.toString());
                }
            }
        } catch (Throwable t) {
            Log.d("getMessages():", t.toString());
        } finally {
            oDB.close();
        }

        if (bGotAMessage) {
            Integer iMessage = 0;
            if (getStatusBarBackgroundLightValue() < 128) {
                iMessage = R.drawable.message_menu;
            } else {
                iMessage = R.drawable.message;
            }
            Intent messageIntent = new Intent(oContext.getApplicationContext(), UserMessages.class);
            PendingIntent messagePendingIntent =
                    PendingIntent.getActivity(
                            oContext.getApplicationContext(),
                            0,
                            messageIntent,
                            PendingIntent.FLAG_UPDATE_CURRENT
                    );
            Notification noti = new Notification.Builder(oContext)
                    .setContentTitle("T.I.S. New Communications")
                    .setContentText("You received " + iMessageCount + " new messages.")
                    .setContentIntent(messagePendingIntent)
                    .setSmallIcon(iMessage).build();

            NotificationManager notificationManager = (NotificationManager) oContext.getSystemService(oContext.NOTIFICATION_SERVICE);

            // hide the notification after its selected
            noti.flags |= Notification.FLAG_AUTO_CANCEL;

            notificationManager.notify(0, noti);
        }
    }

    private static int lastMsgIdSync() {
        int iRtnVal = 0;

        // put the information into the database
        TISDBHelper oDB = new TISDBHelper(oContext, sDBName, null, iVersionVal);
        try {
            oDB.createDataBase();
        } catch (IOException ioe) {
            throw new Error("Unable to create database");
        }

        oDB.openDataBase();
        String sSQL = "select max(MsgID) from Messages";
        ArrayList<ArrayList<String>> sStringList = oDB.selectRecordsFromDBList(
                sSQL, null);
        oDB.close();

        if (sStringList != null) {
            for (int i = 0; i < sStringList.size(); i++) {
                int iSize = sStringList.size();
                ArrayList<String> tempList = sStringList.get(i);
                try {
                    iRtnVal = Integer.parseInt(tempList.get(0));
                } catch (Exception ex) {
                    iRtnVal = 0;
                }
            }
        } else {
            iRtnVal=0;
        }

        return iRtnVal;
    }

    public static Boolean sendMessage(MessagesClass messagesClass) throws Exception {
        Boolean bRtnVal = false;

        DBParameters oDBParameters = new DBParameters();

        // WebServer Request URL
        String serverURL = oDBParameters.readParameter(
                "REST_BASE_URL", oContext, sDBName,
                iVersionVal);

        serverURL += "user_message_save_ws.php";

        String data = "";

        try {
            // Set Request parameter
            data += "&" + URLEncoder.encode("fromUser", "UTF-8") + "=" + messagesClass.FromUserName;
            data += "&" + URLEncoder.encode("inMessage", "UTF-8") + "=" + messagesClass.Message;
            data += "&" + URLEncoder.encode("toUserID", "UTF-8") + "=" + messagesClass.ToUserName;
            data += "&" + URLEncoder.encode("inMsgTime", "UTF-8") + "=" + messagesClass.MsgDate;
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        BufferedReader reader = null;
        StringBuilder sb = new StringBuilder();
        // Send data
        try {

            // Defined URL  where to send data
            URL url = new URL(serverURL);

            // Send POST data request
            URLConnection conn = url.openConnection();
            conn.setDoOutput(true);
            OutputStreamWriter wr = new OutputStreamWriter(conn.getOutputStream());
            wr.write(data);
            wr.flush();

            // Get the server response
            reader = new BufferedReader(new InputStreamReader(conn.getInputStream()));
            String line = null;

            // Read Server Response
            while ((line = reader.readLine()) != null) {
                // Append server response in string
                sb.append(line + "");

            }
        } catch (Exception ex) {
            ex.printStackTrace();
            Log.e("SyncMessages Error:", ex.toString());
        } finally {
            try {
                reader.close();
            } catch (Exception ex) {
            }
        }

        // put the information into the database
        TISDBHelper oDB = new TISDBHelper(oContext, sDBName, null, iVersionVal);
        try {
            oDB.createDataBase();
        } catch (IOException ioe) {
            throw new Error("Unable to create database");
        }

        oDB.openDataBase();

        String sId = "";
        try {
            JSONArray oJsonArray = new JSONArray(sb.toString());

            for (int iCounter = 0; iCounter < oJsonArray.length(); iCounter++) {

                JSONObject json = oJsonArray.getJSONObject(iCounter);
                sId =  json.getString("id");

                ContentValues oValues = new ContentValues();
                oValues.put("MsgID", sId);

                // update the MSgID in the data for the message
                try {
                    oDB.updateRecordsInDB("Messages", "_id=" + messagesClass.id, oValues);
                    bRtnVal = true;
                } catch (Exception sqlEx) {
                    Log.e("sqlEx:", sqlEx.toString());
                }
            }
        } catch (Throwable t) {
            Log.d("getMessages():", t.toString());
        } finally {
            oDB.close();
        }

        return bRtnVal;
    }

    public static Boolean sendUpdStatus(MessagesClass messagesClass) throws Exception {
        Boolean bRtnVal = false;

        DBParameters oDBParameters = new DBParameters();

        // WebServer Request URL
        String serverURL = oDBParameters.readParameter(
                "REST_BASE_URL", oContext, sDBName,
                iVersionVal);

        serverURL += "updMsgStatusWS.php";

        String data = "";

        try {
            // Set Request parameter
            data += "&" + URLEncoder.encode("msgStatus", "UTF-8") + "=" + messagesClass.MsgStatus;
            data += "&" + URLEncoder.encode("msgID", "UTF-8") + "=" + messagesClass.MsgID;
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        BufferedReader reader = null;
        StringBuilder sb = new StringBuilder();
        // Send data
        try {

            // Defined URL  where to send data
            URL url = new URL(serverURL);

            // Send POST data request
            URLConnection conn = url.openConnection();
            conn.setDoOutput(true);
            OutputStreamWriter wr = new OutputStreamWriter(conn.getOutputStream());
            wr.write(data);
            wr.flush();

            // Get the server response
            reader = new BufferedReader(new InputStreamReader(conn.getInputStream()));
            String line = null;

            // Read Server Response
            while ((line = reader.readLine()) != null) {
                // Append server response in string
                sb.append(line + "");

            }
            bRtnVal = true;
        } catch (Exception ex) {
            ex.printStackTrace();
            Log.e("SyncMsgStatus Error:", ex.toString());
        } finally {
            try {
                reader.close();
            } catch (Exception ex) {
            }
        }

        if (bRtnVal == true && messagesClass.MsgStatus.toUpperCase().equals("DELETE")) {
            try {
                TISDBHelper oDB = new TISDBHelper(oContext, sDBName, null, iVersionVal);
                try {
                    oDB.createDataBase();
                } catch (IOException ioe) {
                    throw new Error("Unable to create database");
                }
                oDB.openDataBase();
                String sWhereClause = "MsgID=" + messagesClass.MsgID;
                oDB.openDataBase();
                oDB.deleteRecordInDB("Messages", sWhereClause);
                oDB.close();
            } catch( Exception ex) {
                DBLogs.insert("Database Exception " + ex.getMessage(),
                        "SyncMessages.sendUpdStatus",
                        oContext, sDBName, iVersionVal);
            }
        }
        return bRtnVal;
    }

    /*
    * Returns estimated value of the lightness of the status bar background
    * @return
            */
    private static int getStatusBarBackgroundLightValue()
    {
        // better this than nothing
        Drawable bg = oContext.getResources().getDrawable(android.R.drawable.status_bar_item_background);
        int height = Math.max(1, bg.getIntrinsicHeight());
        int width = Math.max(1, bg.getIntrinsicWidth());
        Bitmap bitmap = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888);
        Canvas canvas = new Canvas(bitmap);
        bg.setBounds(0, 0, width, height);
        bg.draw(canvas);

        long sum = 0;
        for (int x=0; x<width; x++){
            for (int y=0; y<height; y++){
                int color = bitmap.getPixel(x, y);
                int r = (color >> 16) & 0xFF;
                int g = (color >> 8) & 0xFF;
                int b = (color) & 0xFF;
                int max = Math.max(r, Math.max(g, b));
                int min = Math.min(r, Math.min(g, b));
                int l = (min + max)/2;
                sum = sum + l;
            }
        }
        bitmap.recycle();
        bitmap = null;
        canvas = null;
        bg = null;
        sum = sum / (width * height);
        // should be [0..255]
        return (int)Math.min(255, Math.max(sum, 0));
    }
}
