package com.exquisiteapplicationsolutions.tis.webServices;

import android.content.Context;

import com.exquisiteapplicationsolutions.tis.DBLogs;
import com.exquisiteapplicationsolutions.tis.DBParameters;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.ProtocolException;
import java.net.URL;

/**
 * Created by jnissley on 8/12/15.
 */
public class SyncJson {
    private Context context;
    private String webserviceName;
    private String sendString = "";  //This should be a json string but does not need to be
    private String baseURL;
    private Boolean success = false;
    private String returnValue;
    public enum Types{
        gps,
        message,
        job,
        dvir
    }
    private Types type = Types.job;
    private static final String sDBName = "TIS.db";
    private static final int iVersionVal = 1;

    // Setters
    public void setContext(Context context) {
        this.context = context;
    }
    public void setWebserviceName(String webserviceName) {
        this.webserviceName = webserviceName;
    }
    public void setSendString(String sendString) {
        this.sendString = sendString;
    }
    public void setBaseURL(String baseURL) {
        this.baseURL = baseURL;
    }
    public void setType(Types type) {
        this.type = type;
    }

    // Getters
    public Boolean getSuccess() {
        return success;
    }
    public String getReturnValue() {
        return returnValue;
    }

    /*
    This will send JSON text or really any text to the web service on the server
     */
    public void SyncData() {
        DBParameters oDBParameters = new DBParameters();
        String serverURL;
        switch (this.type) {
            case message:
            case job:
            case dvir:
                serverURL = oDBParameters.readParameter(
                        "REST_BASE_URL", this.context, sDBName,
                        iVersionVal);
                break;
            case gps:
                serverURL = oDBParameters.readParameter(
                        "REST_GPS_BASE_URL", this.context, sDBName,
                        iVersionVal);
                break;
            default:
                serverURL = oDBParameters.readParameter(
                        "REST_BASE_URL", this.context, sDBName,
                        iVersionVal);
                break;
        }
        String jsonString = "";
        try {
            URL url = new URL(serverURL + this.webserviceName);
            HttpURLConnection con = (HttpURLConnection) url.openConnection();
            try {
                con.setRequestMethod("POST");
                con.setDoOutput(true);
                con.setDoInput(true);
                con.setInstanceFollowRedirects(false);
                con.setRequestProperty("Content-Type", "application/json; charset=UTF-8");
                con.setRequestProperty("X-Requested-With", "XMLHttpRequest");
                OutputStream out = new BufferedOutputStream(con.getOutputStream());
                out.write(this.sendString.getBytes("UTF-8"));
                out.flush();
                out.close();
                if (con.getResponseCode() == 200) {
                    this.success = true;
                    BufferedReader reader = new BufferedReader(new InputStreamReader(con.getInputStream()));
                    String sLine = null;
                    StringBuilder sb = new StringBuilder();
                    // Read Server Response
                    while ((sLine = reader.readLine()) != null) {
                        // Append server response in string
                        sb.append(sLine + "");
                    }
                    jsonString = sb.toString();
                    this.returnValue = jsonString;
                } else {
                    //Bad response from server
                    this.success = false;
                    DBLogs.insert("Connection to web server failed with code " + con.getResponseCode(),
                            "SyncJson Class",
                            this.context, sDBName, iVersionVal);
                }
            } catch (ProtocolException pex) {
                this.success = false;
                /*DBLogs.insert("Protocol Exception " + pex.getMessage(),
                        "SyncJson Class",
                        this.context, sDBName, iVersionVal);*/
            }
        } catch (IOException ioex) {
            this.success = false;
            /*DBLogs.insert("IO Exception " + ioex.getMessage(),
                    "SyncJson Class",
                    this.context, sDBName, iVersionVal);*/
        }
        // Can not do this yet because not all responses are in json
        /*if (this.success) {
            //Now test if the JSON is valid
            if (isJSONValid(jsonString)) {
                this.returnValue = jsonString;
            } else {
                this.success = false;
            }
        }*/
    }

    private Boolean isJSONValid(String sValue) {
        // Checking if string is really json object or array
        Boolean bRTNValue;
        try {
            new JSONObject(sValue);
            bRTNValue = true;
        } catch (JSONException ex) {
            try {
                new JSONArray(sValue);
                bRTNValue = true;
            } catch (JSONException ex1) {
                bRTNValue = false;
            }
        }
        return bRTNValue;
    }
}
